/**
* CArray ist eine einfache Integer-Array-Klasse
* @author : Mirko Mumberg
* @version : 1.0  date 07.06.1997
*/
public class CArray
{
	/**
	* Ist das Feld selbst.
	*/
	protected int [] m_Array;
	/**
	* Ist die Groesse des Feldes.
	*/
	protected int m_Size;
	/**
	* Die Lese- und Schreibzugriffszaehler.
	*/
	protected CCounter m_GetCount, m_SetCount;

	/**
	* Konstruktor, der das Array mit der Groesse und den Zaehlern initialisiert. 
	* @param _Size Die neue Groesse des Arrays.
	* @param _Get Der Getzaehler.
	* @param _Set Der Setzaehler.
	*/
	public CArray(int _Size, CCounter _Get, CCounter _Set)
	{
		init(_Size, _Get, _Set);
	}

	/**
	* Konstruktor, der das Array mit der Groesse initialisiert ohne Schreib- und Lesezugriffszaehler. 
	* @param _Size Die neue Groesse des Arrays.
	*/
	public CArray(int _Size)
	{
		init(_Size, null, null);
	}

	/**
	* Der Kopiekonstruktor.
	* @param _a Das Array, das in dieses kopiert werden soll.
	*/
	public CArray(CArray _a)
	{
		init(_a.m_Size, _a.m_GetCount, _a.m_SetCount);
		copy(_a);
	}

	/**
	* Initialisierung des Arrays.
	* @param Size Die neue Groesse des Arrays.
	* @param Get Der Getzaehler.
	* @param Set Der Setzaehler.
	*/
	public void init(int Size, CCounter Get, CCounter Set)
	{
		m_Size = Size;
		m_Array = new int[Size];
		m_GetCount = Get;
		m_SetCount = Set;
	}

	/**
	* Die Kopierfunktion.
	* @param a Das Array, das in dieses kopert werden soll.
	*/
	public void copy(CArray a)
	{
		System.arraycopy(a.m_Array, 0, m_Array, 0, a.m_Array.length);
		m_GetCount = a.m_GetCount;
		m_SetCount = a.m_SetCount;
	}

	/**
	* Veraenderung der Groesse des Arrays.
	* @param Size Die neue Groesse des Arrays. 
	*/
	public void setSize(int Size)
	{
		m_Size = Size;
		m_Array = new int[Size];
		if (m_GetCount != null) m_GetCount.set(0);
		if (m_SetCount != null) m_SetCount.set(0);
	}

	/**
	* Setzen eines Wertes in diesen Array.
	* @param pos Position, an der der Wert gespeichert wird.
	* @param x Der Wert, der gespeichert wird.
	*/
	public synchronized void Set(int pos, int x)
	{
		if((pos >= 0) && (pos < m_Size))
		{
			m_Array[pos] = x;
			if (m_SetCount != null) m_SetCount.up();
		}
		else System.out.println("Fehlerhafter Array Zugriff" + pos);
	}

	/**
	* Einen Wert aus dem Array lesen.
	* @param pos Position, aus der der Wert gelesen wird.
	* @return Der Wert an der Position pos
	*/
	public synchronized int Get(int pos)
	{
		if((pos >= 0) && (pos < m_Size))
		{
			if (m_GetCount != null) m_GetCount.up();
			return m_Array[pos];
		}
		else
		{
			System.out.println("Fehlerhafter Array Zugriff" + pos);
			return 0;
		}
	}

	/**
	* Die groesse des Arrays wird gelesen.
	* @return die groesse des Arrays.
	*/
	public int getSize()
	{
		return m_Size;
	}
}

