import java.awt.*;
import java.awt.image.*;

/**
* CPzzleElement ist ein Kontainer fuer ein Puzzelteil mit seinen 4 moeglichen Darstellungen.
*
* @author Mirko Mumberg
* @version 1.0  date 07.06.1997
*/
public class CPuzzleElement extends Canvas
{
	private Image 	m_Element[];		// Das Array fuer die 4 Images des Puzzelteils.
	private int 	m_width, m_height;	// Breite und Hoehe des Puzzelteils.
	private int 	m_NewImageArray1[];	// Integer-Array fuer die Erstellung der weitern Puzzelteile.
	private int 	m_NewImageArray2[];	// Integer-Array fuer die Erstellung der weitern Puzzelteile.
	private int 	m_NewImageArray3[];	// Integer-Array fuer die Erstellung der weitern Puzzelteile.
	private int 	m_OldImageArray[];	// Integer-Array fuer die Erstellung der weitern Puzzelteile.

	/**
	* Kontruktor initialisiert die 4 moeglichen Darstellungen.
	* @param img Image eines Puzzelteils.
	*/
	public CPuzzleElement(Image _img)
	{
		m_Element = new Image[4];
		m_Element[0] = _img;
		m_width = m_Element[0].getWidth(this);
		m_height =m_Element[0].getHeight(this);
		m_NewImageArray1  = new int[m_width*m_height];
		m_NewImageArray2  = new int[m_width*m_height];
		m_NewImageArray3  = new int[m_width*m_height];
		m_OldImageArray = new int[m_width * m_height];
		makeImages();
		m_NewImageArray1 = null;
		m_NewImageArray2 = null;
		m_NewImageArray3 = null;
		m_OldImageArray = null;
	}

	/**
	* Wandelt ein Image in ein Integer-Array um
	* @param img Das Source-Image.
	* @param pix Das Integer-Array.
	*/
	protected void grab(Image img, int pix[])
	{
		PixelGrabber grabber = new PixelGrabber(img, 0, 0,m_width, m_height, pix, 0, m_width);
		try
		{
			grabber.grabPixels();
		}
		catch (InterruptedException e)
		{
			System.err.println("Error: "+e);
		}
	}

	/**
	* Wandelt ein Integer-Array in ein Image um.
	* @param pix Das Source-Integer-Array.
	* @return Das neue Image.
	*/
	protected Image doImage(int[] pix)
	{
		return createImage(new MemoryImageSource(m_width, m_height,pix, 0, m_width));
	}

	/**
	* Erstellt aus dem ersten Image (das dem Konstruktor uebergeben wurde) die 3 weiteren Images. Diese Images sind jeweils um 90 Grad gedreht.
	*/
	protected void makeImages()
	{
		grab(m_Element[0], m_OldImageArray);
		int k = 0;
		for(int i = 0; i < m_width; i++)
		{
			for(int j = m_height-1; j > -1; j--)
			{
				m_NewImageArray1[k] = m_OldImageArray[j*m_width +i];
				m_NewImageArray2[(m_height-j-1)*m_width + (m_width-i-1)] = m_OldImageArray[j*m_width +i];
				m_NewImageArray3[(m_width*m_height-1)-k++] = m_OldImageArray[j*m_width +i];
			}
		}
		m_Element[1] = doImage(m_NewImageArray1);
		m_Element[2] = doImage(m_NewImageArray2);
		m_Element[3] = doImage(m_NewImageArray3);
	}

	/**
	* Gibt das mit i ausgewaehlte Image zurueck.
	* @param i Die Nummer des gewuenschten Image.
	* @return Das gewuenschte Image.
	*/
	public Image getElement(int i)
	{
		return m_Element[i];
	}
}

