import java.awt.*;
/**
* CSortCanvas ist die grafische Oberflaeche, auf der das Array dargestellt wird.
* Beinhaltet die Schnittstelle Runnable, um die Funktionen eines Threat zu bekommen.
* In dem Thread wird die Sortierung durchgefuehrt.
*
* @author Mirko Mumberg
* @version 1.0  date : 07.06.1997 
*/
public class CSortCanvas extends Canvas
{
	private Dimension 	m_dPrefSize;  	// Groesse des Canvas
	private CArrayRandom	m_VisualArray;	// Zeiger auf das Sortier-Array
	private int		m_ElementSize;  // Groesse der Quadratekante in Pixel.
	private CSortAlg 	m_SortAlg;		// Der Sortier-Algorithmuss
	private Image		m_imageBuffer;  	// Um im Hintergrund den Arrayinhalt zu Zeichen.
	private Graphics 	m_graphicsBuffer;	// Und Blinkfrei auf den Bildschirm zu bekommen.
	private CCounter	m_CounterCompare, m_CounterGets, m_CounterSets; // Die Statistik-zaehler
	
	/**
	* Konstruktor initialisiert die Zeichenflaeche, setzt die Counter auf 0 und erstellt ein erstes Array zum sorieren.
	* @param _Size Die Groesse des Arrays.
	* @param _Get Der Counter, der die Get-Zugriffe auf das zusortierende Array zaehlt.
	* @param _Set Der Counter, der die Set-Zugriffe auf das zusortierende Array zaehlt.
	* @param _Compare Der Counter, der die Vergleiche bei den SortierVerfahren zaehlt.
	*/
	public CSortCanvas(int _Size, CCounter _Get, CCounter _Set, CCounter _Compare)
	{
		setForeground(Color.black);
		setBackground(Color.white);
		m_VisualArray = new CArrayRandom(_Size, _Get, _Set);
		m_VisualArray.InitDifferentNumbers();
		m_CounterCompare = _Compare;
		m_CounterGets = _Get;
		m_CounterSets = _Set;
		m_ElementSize = 7;
		setFirstSize(_Size);
		m_CounterCompare.set(0);
		m_CounterGets.set(0);
		m_CounterSets.set(0);
	}

	/**
	* Initialisiert den Hintergrundspeicher (muss vor StartSort aufgerufen werden).
	*/
	public void init()
	{
		m_imageBuffer = createImage(getSize().width, getSize().height);
		m_graphicsBuffer = m_imageBuffer.getGraphics();
	}

	/**
	* Um die Groesse beizubehalten.
	* @return Gibt die Groesse in Dimensionen zurueck.
	*/
	public Dimension getPreferredSize()
	{
		return m_dPrefSize;
	}

	/**
	* Um die Groesse bezubehalten.
	* @return Gibt die Groesse in Dimensionen zurueck.
	*/
	public Dimension getMinimumSize()
	{
		return m_dPrefSize;
	}

	/**
	* Veraendert die Groesse des Arrays und des Canvas. Das Array wird ohne doppelte Werte intitialisiert. Die Groesse des Hintergrundspeichers wird angepasst. Die Counter werden auf 0 gesetzt.
	* @param _Size Die neue Groesse des Arrays.
	*/
	public void setSize(int _Size)
	{
		setFirstSize(_Size);
		m_imageBuffer = createImage(m_dPrefSize.width, m_dPrefSize.height);
		m_graphicsBuffer = m_imageBuffer.getGraphics();
		repaint();
	}

	/**
	* Veraendert die Groesse des Arrays und des Canvas. Das Array wird ohne doppelte Werte intitialisiert. Die Counter werden auf 0 gesetzt.
	* @param _Size Die neue Groesse des Arrays.
	*/
	private void setFirstSize(int _Size)
	{
		m_dPrefSize = new Dimension( _Size * m_ElementSize, _Size * m_ElementSize);
		super.setSize(m_dPrefSize);
		m_VisualArray.setSize(_Size);
		m_VisualArray.InitDifferentNumbers();
		m_CounterSets.set(0);
		m_CounterGets.set(0);
		m_CounterCompare.set(0);
	}

	/**
	* Setzen der Pixelanzahl der Quadratekanten, die ide Arrayelemente darstellen.
	* @param Pixel Die Pixelanzahl.
	*/
	public void setElementSize(int pixel)
	{
		m_ElementSize = pixel;
		setSize(m_VisualArray.getSize());
	}


	/**
	* Setzen des Sortieralgorithmuss.
	* @param sortalg Der zu setzende Sortieralgorithmuss.
	*/
	public void setSort(CSortAlg sortalg)
	{
		m_SortAlg = sortalg;
	}

	/**
	* Sofortiges neuzeichnen des Array.
	* Zuerst wird auf ein nicht sichbaren Image gezeichnet. Dieses Image wird dann ueber das aktuelle kopiert. Dadurch wird flimmern der Grafikflaeche vermieden.
	*/
	public void paint()
	{
     		update(getGraphics());
	}

	/**
	* Zeichnen des Array am Anfang des Applet, Applikation.
	* @param g Die Graphikflaeche zum Zeichnen.
	*/
	public void paint(Graphics g)
	{
		PaintArray(g);
	}

	/**
	* Neuzeichnen des Array. Ausgeloest durch ein repaint-Aufruf.
	* Zuerst wird auf ein nicht sichbaren Image gezeichnet. Dieses Image wird dann ueber das aktuelle kopiert. Dadurch wird flimmern der Grafikflaeche vermieden.
	* @param g Die sichtbare Graphikflaeche zum Zeichnen.
	*/
	public void update(Graphics g)
	{
		m_graphicsBuffer.setColor(getBackground());
		m_graphicsBuffer.fillRect(0, 0, getSize().width, getSize().height);
		m_graphicsBuffer.setColor(getForeground());
		PaintArray(m_graphicsBuffer);
		g.drawImage(m_imageBuffer, 0, 0, this);
	}

	/**
	* Zeichnen der einzelnen Array-Quadrate.
	* Dabei wird der Get-Counter berichtigt, um nicht die Gets des Zeichnens mitzuzaehlen.
	* @param g Die Grafikflaeche zum Zeichen.
	*/
	public void PaintArray(Graphics g)
	{
		for(int i = 0; i < m_VisualArray.getSize(); i++)
		{
			g.fillRect(i * m_ElementSize, m_VisualArray.Get(i) * m_ElementSize, m_ElementSize, m_ElementSize);
			m_CounterGets.down();
		}
	}

	/**
	* Die Counter werden mit 0 initialisiert.
	* Der Sortieralgorithmuss-Klasse wird der CSortCanvas uebergeben und die Sortierung wird gestartet.
	*/
	public void StartSort()
	{
		m_CounterCompare.set(0);
		m_CounterGets.set(0);
		m_CounterSets.set(0);
		m_SortAlg.setParent(this, m_CounterCompare);
		m_SortAlg.Sort(m_VisualArray);
	}
}

