import java.awt.event.*;
import java.awt.*;
/**
* CSortMenuBar stammt von MenuBar ab und beinhaltet das Menue mit den Sortiermoeglichkeiten.
* Um die Aktivitaeten auszufuehren wird das Interface Actionlistener implementiert.
*
* @author Mirko Mumberg
* @version 1.0  date : 07.06.1997 
*/
public class CSortMenuBar extends MenuBar implements ActionListener
{
	private Frame		m_SortFrame; 		// m_SortFrame ist das Frame, das durch den Beenden-Punkt beendet wird.
	private Menu 		m_SortMenu;		// m_SortMenu ist das Pulldownmenue.
	private CSortCanvas 	m_SortCanvas;		// m_SortCanvas ist das CSortCanvas, in dem die Sortierung ausgefuehrt werden soll.
	private Label		m_SortierMethode;	// m_SortMethode der Label, auf dem die aktuelle Sortiermethode ausgegeben werden soll.

	/**
	* Der Konstruktor initialisiert das Menue und fuegt das Interface dazu.
	* @param SCanvas Ist ein CSortCanvas. In dem wird die Sortierung durchgefuehrt.
	* @param SFrame Ist ein Frame. Dieses Frame wird durch den Beenden-Punkt beendet.
	*/
	public CSortMenuBar(CSortCanvas _SCanvas, Frame _SFrame, Label _SMethode)
	{
		super();
		m_SortCanvas = _SCanvas;
		m_SortFrame = _SFrame;
		m_SortierMethode = _SMethode;
		m_SortMenu = new Menu("Sortieren");
		m_SortMenu.addActionListener(this);
		m_SortMenu.add("Quicksort");
		m_SortMenu.add("Bubblesort");
		m_SortMenu.add("Selectionsort");
		m_SortMenu.add("Insertionsort");
		m_SortMenu.add("Heapsort");
		m_SortMenu.add("Mergesort");
		m_SortMenu.add("Beenden");
		add(m_SortMenu);
	}

	/*
	* Stop einen alten SortierThreat und startet das ausgewaehlten Sortierverfahren.
	* @param SortAlg Die SortierAlgorithmuss-Klasse, mit der die Sortierung durchgefuehrt werden soll.
	*/
	private void SortStart(CSortAlg SortAlg)
	{
		m_SortCanvas.setSort(SortAlg);
		m_SortCanvas.init();
		m_SortCanvas.StartSort();
	}

	/**
	* Funktion aus der Schnittstelle ActionListener.
	*
	* Wird aufgerufen, wenn im Menue ein Punkt ausgewaehlt wird. 
	* Fuehrt das Sortieren und das Beenden aus aus.
	* @param e Event, das den Menuepunkt darstellt. 
	*/
	public void actionPerformed(ActionEvent e)
	{
		String cmd = e.getActionCommand();
		if(cmd.equals("Quicksort"))
		{
			m_SortierMethode.setText("Quicksort");
			CQuicksortAlg Sort;
			Sort = new CQuicksortAlg();
			SortStart(Sort);
		}
		if(cmd.equals("Bubblesort"))
		{
			m_SortierMethode.setText("Bubbleort");
			CBubblesortAlg Sort;
			Sort = new CBubblesortAlg();
			SortStart(Sort);
 
		}
		if(cmd.equals("Selectionsort"))
		{
			m_SortierMethode.setText("Selectionsort");
			CSelectionsortAlg Sort;
			Sort = new CSelectionsortAlg();
			SortStart(Sort);
		}
		if(cmd.equals("Insertionsort"))
		{
			m_SortierMethode.setText("Insertionsort");
			CInsertionsortAlg Sort;
			Sort = new CInsertionsortAlg();
			SortStart(Sort);
		}
		if(cmd.equals("Heapsort"))
		{
			m_SortierMethode.setText("Heapsort");
			CHeapsortAlg Sort;
			Sort = new CHeapsortAlg();
			SortStart(Sort);
		}
		if(cmd.equals("Mergesort"))
		{
			m_SortierMethode.setText("Mergesort");
			CMergesortAlg Sort;
			Sort = new CMergesortAlg();
			SortStart(Sort);
		}
		if(cmd.equals("Beenden"))
		{
			m_SortFrame.dispose();
			System.exit(0);
		}
	}
	
}

