import java.awt.*;
import java.awt.event.*;
/**
* CStepChoice ist ein Choice, dessen Elemente sich nur als Werte mit konstanten Abstaenden
* unterscheiden. Diese Klasse ist abstrakt. Das Interface ItemListener wird implementiert um nach einer Auswahl die Action
* auszufuehren. Von dieser Klasse sollte fuer die Verarbeitung eine Klasse abgeleitet werden und die Methode doAction
* mit den Aktivitaeten implementiert werden.
*
* @author Mirko Mumberg
* @version 1.0  date : 07.06.1997 
*/
public abstract class CStepChoice extends Choice implements ItemListener
{
	/**
	* Label 		sind die einzelnen Choice-Strings.
	*/
	protected String [] m_Label;
	/**
	* CountItems 	ist die Anzahl der Choiceelemente.
	*/
	protected int m_CountItems;
	/**
	* Begin 		ist der Anfangswert.
	*/
	protected int m_Begin;
	/**
	* Step 		ist die Schrittweite.
	*/
	protected int m_Step;

	/**
	* Der Konstructor initialisiert den Choice und fuegt den ItemListener dazu.
	* @param _Begin Ist der unterste Wert im Choice
	* @param _End Ist der oberste Wert im Choice
	* @param _Step Ist die Schrittgroesse zwischen den Werten im Choice
	* @param _Default Ist das Defaultelement. Es enthaelt die Position des Defaultelements. Begonnen wird mit 0 fuer das Beginelement.
	* @param _Label Ist der String der an den Wert angehaengt wird. 
	*/
	public CStepChoice(int _Begin, int _End, int _Step, int _Default, String _Label)
	{
		m_Begin = _Begin;
		m_Step = _Step;
		m_CountItems = (_End - _Begin) / m_Step + 1;
		m_Label = new String[m_CountItems];
		for(int i = 0; i < m_CountItems; i++)
		{
			m_Label[i] = (m_Begin + i * m_Step) + _Label;
			addItem(m_Label[i]);
		}
		select(_Default);
		addItemListener(this);
	}
	
	/**
	* Funktion aus der Schnittstelle ItemListener.
	* Wird aufgerufen, wenn im Choice-Element eine Auswahl getroffen wird.
	* Fuehrt die Funktion doAction, mit dem Wert als Parameter, aus.
	* @param e Event, das vom Choice gesendet wird.
	*/
	public void itemStateChanged(ItemEvent e)
	{
		String cmd = e.getItem().toString();
		boolean found = false;
		int i = 0;

		while (!found && i < m_CountItems)
		{
			if(cmd.equals(m_Label[i]))
			{
				doAction(m_Begin + i * m_Step);
				found = true;
			}
			i++;
		}
	}

	/**
	* Ist eine abstrakte Funktion, die nach der Auswahl im Choice ausgefuehrt wird.
	* @param Value Ist der Wert, der im Choice ausgewaehlt wurde (Ohne String-anhang). 
	*/
	public abstract void doAction(int Value);
}

