import java.awt.Color;	// Graphics Interface

import Coordinate;		// coordinate class
import Light;			// light class

/**
 * ColorObject.java:	interface-class
 *						defines the interface that every ColorObject hast to implement
 *
 * Author:  Sven Sa 
 * on:		06/06/97
 */
public interface ColorObject
{
	/**
	 * calculates the color relative to position and normal vector
	 *
	 * @param pos position of faces
	 * @param normal normal vector on face
	 * @return Color the current color
	 */
	public abstract Color	lightning (Coordinate pos, Coordinate normal);

	/**
	 * calculates the color relative to position and normal vector
	 *
	 * @param plane position and normal of faces
	 * @return Color the current color
	 */
	public abstract Color	lightning (Light plane);

	/**
	 * adds another lightsource to the calculation
	 *
	 * @param light position and direction of lightsource
	 * @return int the index of lightsource
	 */
	public abstract int		addLight (Light light);

	/**
	 * adds another lightsource to the calculation
	 *
	 * @param pos position of lightsource 
	 * @param dir direction of lightsource
	 * @return int the index of lightsource
	 */
	public abstract int		addLight (Coordinate pos, Coordinate dir);
	
	/**
	 * removes a lightsource from the calculation
	 *
	 * @param index index of lightsource as returned by addLight
	 * @return boolean	TRUE lightsource has been removed<br>
	 *					FALSE lightsource has <b>not</b> been removed
	 */
	public abstract boolean removeLight (int index);

	/**
	 * removes all lightsources from the calculation
	 *
	 * @param void
	 * @return void
	 */
	public abstract void	removeAllLights ();

	/**
	 * replaces a lightsource in the calculation
	 *
	 * @param index index of lightsource that has to be replaced
	 * @param light lightsource with which to replaced
	 * @return boolean	TRUE lightsource has been removed<br>
	 *					FALSE lightsource has <b>not</b> been removed
	 */
	public abstract boolean	replaceLight (int index, Light light);

	/**
	 * replaces a lightsource in the calculation
	 *
	 * @param index index of lightsource that has to be replaced
	 * @param pos position of lightsource with which to replaced
	 * @param dir direction of lightsource with which to replaced
	 * @return boolean	TRUE lightsource has been removed<br>
	 *					FALSE lightsource has <b>not</b> been removed
	 */
	public abstract boolean replaceLight (int index, Coordinate pos, Coordinate dir);
	
	/**
	 * sets the falloff value (objects in distance are darker)
	 *
	 * @param fall falloff value
	 * @return void
	 */	
	public abstract void	setFallOff (int fall);

	/**
	 * returns the falloff value
	 *
	 * @param void
	 * @return int falloff value
	 * @see setFallOff
	 */	
	public abstract int		getFallOff();
}
