import java.awt.Graphics;	// graphics interface
import java.awt.Polygon;	// polygon interface
import java.awt.Color;		// color interface
import Coordinate;			// coordinates
import FastRotate;			// rotationobject
import NormalColor;			// colorobject
import FaceDefinition;		// Facedefinition

/**
 * Genera.java:		example of a generic vector object
 *
 * @author Sven Sa 
 * @version 18/10/97
 * @desc    shows how to build up a generic complex object
 */
public class General implements VectorObject
{
private static int cons=0;
private static boolean debug=false;


	private int dist =  512;	// Default Abstand des Objektes DEBUG!!
	private int eye  =  512;	// Augenabstand
	private int mult = 2048;  // Berechnungsgenauigkeit
	private int x_off = 320;
	private int y_off = 200;

	//- Variablen --------------------------------------------------------
	private final static int totalVertices = 3;
	private final static int totalFaces = 1;

	private Coordinate [] rot_punkte;								// the rotated points
	private static Coordinate [] normals;							// the normal Vectors
	private Coordinate [] rot_normals;								// the rotated normal Vectors
	
	private Polygon		  per_punkte;								// the perpectived points
	private Polygon		  current_pts;								// the currenty polygon

	private Coordinate	  rot;										// current rotation
	private Coordinate	  pos;										// current position
	private int hidden = VO_IS_NOT_HIDDEN;

	private FastRotate fastrot;										// the rotation"machine"
	private NormalColor nc;											// the color calculation 

	private final static FaceDefinition [] faces = { new FaceDefinition(0,1,2) };
	private final static int size = 150;
	private final static Coordinate [] punkte = { new Coordinate(-1*size/2,-1*size/2,0),  
												  new Coordinate( 1*size/2,-1*size/2,0),
												  new Coordinate( 1*size/2, 1*size/2,0) };
	
	/**
	 * initializes one object
	 *
	 * @param _punkte the coordinates of the plane
	 * @param col color of the plane
	 * @return void
	 */
	public General (int size)
	{
		try
		{
			generateTables();
		}
		catch (IndexOutOfBoundsException e)
		{
			System.err.println ("Initialisierungsfehler: Nicht ausreichend Punkte fr Konstruktor(General)");
		}

	}
	
	/**
	 * not used
	 * @return always -1 (object is final)
	 */
	public int addItem(VectorObject  obj)
	{
		return -1; // Objekt ist final
	}

	/**
	 * not used
	 * @return always false (object is final)
	 */
	public boolean deleteItem (int item)
	{
		return false; // kann keine Objekt entfernen
	}

	/**
	 * retunrs number of subobjects
	 * @param void
	 * @return always 1 (face has always one face)
	 */
	public int itemCount ( )
	{
		return totalFaces;	
	}

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (int x_rot,int y_rot, int z_rot )
	{
		rot.x += x_rot;
		rot.y += y_rot;
		rot.z += z_rot;
		calculate();
	};

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (Coordinate rot)
	{
		rotate (rot.x,rot.y,rot.z);
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (int x_rot,int y_rot, int z_rot )
	{
		rot.x=x_rot;
		rot.y=y_rot;
		rot.z=z_rot;
		calculate();
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (Coordinate _rot)
	{
		rot = _rot;
		calculate();
	};

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 * @see VectorObject
	 */
	public Coordinate getRotation ( )
	{
		return rot;
	}

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (int x_mov,int y_mov, int z_mov )
	{
		pos.x += x_mov;
		pos.y += y_mov;
		pos.z += z_mov;
	};

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (Coordinate mov)
	{
		translate (mov.x,mov.y,mov.z);
	};

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (int x_mov,int y_mov, int z_mov )
	{
		pos.x = x_mov;
		pos.y = y_mov;
		pos.z = z_mov;
	};

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (Coordinate mov)
	{
		pos = mov;
	};

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 * @see VectorObject
	 */
	public Coordinate getTranslation ( )
	{
		return pos;
	}

	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 * @see VectorObject
	 */
	public void hide ( )
	{
		hidden = VO_IS_HIDDEN;	
	};
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void show ( )
	{
		hidden = VO_IS_NOT_HIDDEN;		
	};

	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void showAlways ( )
	{
		hidden = VO_NEVER_HIDDEN;		
	};

	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 * @see VectorObject
	 */
	public int isHidden ( )
	{
		return hidden;	
	};

	/**
	 * returns average depth of object
	 *
	 * @param void
	 * @return int depth of objectktes
	 * @see VectorObject
	 */
	public int getDepth ( )
	{
		return pos.z;
	};

	/**
	 * draws an object on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 * @see VectorObject
	 */
	public void draw ( Graphics graph )
	{
	   if (hidden == VO_IS_NOT_HIDDEN || hidden == VO_NEVER_HIDDEN)
	   {

		// Calculate normals
		for (int i=0;i<totalFaces; ++i)
		{
			graph.setColor(nc.lightning(pos,normals[i]));			// calculate color

/*			for (int j=0;j<3; ++j)
			{
				current_pts.xpoints[j]= per_punkte.xpoints[faces[i].p[j]];	// the perpectived points
				current_pts.ypoints[j]= per_punkte.ypoints[faces[i].p[j]];
			}*/
			graph.fillPolygon(per_punkte); //current_pts);							// draw polygon
		}

	   } // is not hidden
	};

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @returns TRUE if perspectived normal vector has positive z-axis value or<br>
     *				 hidden flag equals VO_NEVER_HIDDEN
	 * @see VectorObject
	 */
	public boolean isVisible()
	{
		return (hidden == VO_IS_NOT_HIDDEN || hidden == VO_NEVER_HIDDEN);
	}
	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces (0<=int<=1)
	 */
	public int getVisibleCount()
	{
		return 0;//(isVisible())?1:0;
	}

	/**
	 * does the internal rotation calculation
	 *
	 * @param void
	 * @return void	 
	 */
	private void calculate()
	{
 		fastrot.setRotation (rot);									// set position
		fastrot.calculate(punkte ,rot_punkte ,totalVertices);		// rotate points (excl. normals)
		fastrot.calculate(normals,rot_normals,totalFaces);			// rotate normals
	
		for (int i=0; i<totalVertices; ++i)							// perspective points(NOT normal!)
		{
    		per_punkte.xpoints[i]=(((rot_punkte[i].x+pos.x)*mult)/((rot_punkte[i].z+pos.z)+dist)*eye)/mult + x_off;
			per_punkte.ypoints[i]=(((rot_punkte[i].y+pos.y)*mult)/((rot_punkte[i].z+pos.z)+dist)*eye)/mult + y_off;
		}
	}

	/**
	 * builds up the internal tables of vertices and faces
	 *
	 * @param void
	 * @return internal tables beeing initialized
	 */
	private void generateTables() throws IndexOutOfBoundsException
	{
	
		// turn on rotation
		fastrot = new FastRotate();
		nc = new NormalColor (Color.lightGray);

		// reserve memory
//		punkte		= new Coordinate[totalVertices+1];
		rot_punkte	= new Coordinate[totalVertices+1];
		normals		= new Coordinate[totalFaces];
		rot_normals = new Coordinate[totalFaces];

		// Calculate normals
		for (int i=0;i<totalFaces; ++i)
		{
			normals[i] = Coordinate.normalize (Coordinate.cross(Coordinate.sub(punkte[faces[i].p[1]],punkte[faces[i].p[0]]),
					                   						    Coordinate.sub(punkte[faces[i].p[1]],punkte[faces[i].p[2]])));
		}
		per_punkte = new Polygon();									// must be initialized
		current_pts = new Polygon();
		per_punkte.npoints=3;
		current_pts.npoints=3;
		rot = new Coordinate();										// current rotation
		pos = new Coordinate();										// current position
		calculate();
	
	}
}

/*
		if (debug)
		{
			System.out.println("");
			System.out.println ("Constructing general " + cons + " times");

			for (int i=0; i<totalFaces; ++i)
			{

				System.out.print ("Face " + i + "=  { ");
				for (int j=0; j<3; ++j)
				{
      				 System.out.print ("( " + punkte[faces[i].p[j]].x+"," + punkte[faces[i].p[j]].y+"," + punkte[faces[i].p[j]].z+")");
					 if (j<2) System.out.print (", ");
				} 
				System.out.println (" }");
			}

		}




if (debug)
		{
			System.out.println("");
			for (int i=0; i<totalFaces; ++i)
			{
				System.out.print ("Per.Face " + i + "=  { ");
				for (int j=0; j<3; ++j)
				{
      				 System.out.print ("( " + per_punkte.xpoints[faces[i].p[j]]+"," + per_punkte.ypoints[faces[i].p[j]]+")" );
					 if (j<2) System.out.print (", ");
				} 
				System.out.println (" }");
			}
		}
*/