//******************************************************************************
// NormalColor.java:	implements an ColorObject
//
// Author:  Sven Sa 
// on:		06/06/97
//******************************************************************************

import java.awt.Color;	// Color class
import Coordinate;		// coordinate (for normal vector)

public class NormalColor implements ColorObject
{
	//- private statics ----------------------------------------------------
	private final static int max_light = 64;		// max amount of lightsources
	private final static int light_size = 64;		// number of colors
	private final static int light_size2 = light_size;		// number of colors
	private final static int diffuse_light = 16;	// minimal lightning
	private final static int min_color=6;			// minimum color referenced
//	private final static int light_phong_angle=light_size/10;

		private final static int light_phong_angle=15;


/*	private static int min;
	private static int max;
	private static boolean changed;
	private static int used[];*/


	//- private members ----------------------------------------------------
	private int		fallOff = 0;			// the FallOff value
	private Color[] LUT;			// die Farb-LookUpTable

	private static int	light_cnt=0;			// no. of lights
	private static Light[] lights;		// the lightsources

	//- constructors -------------------------------------------------------

	public NormalColor()
	{
		lights = new Light[max_light];		// array for lights
		light_cnt=0;
	}

	public NormalColor (Color col)
	{
		lights = new Light[max_light];		// array for lights
		float red,green,blue;
		
		LUT = new Color[light_size];
		for (int i=0; i<light_size-light_phong_angle;++i)	
		{
			red   = ((((float)col.getRed  ())/256)/light_size) *i; // calculate colors
			green = ((((float)col.getGreen())/256)/light_size) *i;
			blue  = ((((float)col.getBlue ())/256)/light_size) *i;
			red  = (red  <((float)diffuse_light)/256)?((float)diffuse_light)/256:red;		// ensure diffuse_light
			green= (green<((float)diffuse_light)/256)?((float)diffuse_light)/256:green;
			blue = (blue <((float)diffuse_light)/256)?((float)diffuse_light)/256:blue;
//System.out.println ("red=" + red + ", green="+green+", blue="+blue);
			LUT [i] = new Color (red,green,blue); 
		}

		// do phong shading
		int j=1;

		float minPhongColor=(float).9;								// with phong shading
		float r=(float)col.getRed()  /256,
			  g=(float)col.getGreen()/256,
			  b=(float)col.getBlue() /256;							// ensure at least a bit
		if (r<minPhongColor) r=(minPhongColor-r)/light_phong_angle;else r=0;						// of color value
		if (g<minPhongColor) g=(minPhongColor-g)/light_phong_angle;else g=0;
		if (b<minPhongColor) b=(minPhongColor-b)/light_phong_angle;else b=0;

		for (int i=light_size-light_phong_angle; i<light_size;++i,++j)	
		{
			red   = ((((float)col.getRed  ()+1)/256)/light_size)*i+r*j; // calculate colors
			green = ((((float)col.getGreen()+1)/256)/light_size)*i+g*j;
			blue  = ((((float)col.getBlue ()+1)/256)/light_size)*i+b*j;

//			System.out.println ("red=" + red + ", green="+green+", blue="+blue);

			if (red  >.99) red  =(float).99;
			if (green>.99) green=(float).99;
			if (blue >.99) blue =(float).99;
/*			red  = (red  <((float)diffuse_light)/256)?((float)diffuse_light)/256:red;		// ensure diffuse_light
			green= (green<((float)diffuse_light)/256)?((float)diffuse_light)/256:green;
			blue = (blue <((float)diffuse_light)/256)?((float)diffuse_light)/256:blue;
*/

			LUT [i] = new Color (red,green,blue); 
			

		}


/*
		min=9999;
		max=-9999;
		changed=true;
		used = new int[light_size];
*/
	};

	//- implmentation ------------------------------------------------------
	private boolean checkRange (int index)
	// checks if the refernced light is existent
	{
		return (index>=0 && index<max_light && index < light_cnt);
	}

	public Color lightning (Light plane)
	// calculates the lightning of a plane depending on normal vector and distance
	{
		return lightning (plane.pos,plane.dir); 
	};

	public Color lightning (Coordinate pos, Coordinate normal)
	// calculates the lightning of a plane depending on normal vector and distance
	{
		int zval,ret;
		if (light_cnt>0)	// any lightsources??
		{
			zval= 0;
			ret = 0;
			for (int i=0;i<light_cnt;++i)
			{
				zval =  -Coordinate.dot (normal,lights[i].dir)* light_size2 /
					       (Coordinate.NORMAL_BASE*Coordinate.NORMAL_BASE);
				if (zval<min_color) zval=min_color;
				if (zval>=light_size) zval=light_size-1;
				ret += zval;
			}
			ret /= light_cnt;

		} else ret=min_color;		// if not return diffuse_light

/*// debugging information (color resolution)		
		if(ret<min){min=ret;changed=true;}
		if (ret>max){max=ret;changed=true;}
		if (changed)System.out.println ("min: " + min + "  max: " + max);
		changed=false;
		if (used[ret]==0)
		{
			used[ret]=1;
			for (int i=0; i<light_size;i++)
			  if (used[i]==1)
			  System.out.println ("Used Color: " + i);
		}
*/
		return LUT[ret];
	};

	public int addLight (Light light)
	// add a new lightsource
	{
		if (light_cnt<max_light)
		{
			Light tl = new Light(light);
			tl.dir = Coordinate.normalize(light.dir);
			lights[light_cnt++] = tl;
			return (light_cnt - 1);
		} else return -1;			
	};

	public int addLight (Coordinate pos, Coordinate dir)
	// add a new lightsource
	{
		return addLight (new Light(pos,dir));
	};

	public boolean removeLight (int index)
	// removes a specific light
	{
		if (checkRange(index))
		{
			for (int i=index; i<light_cnt;++i)	lights[i]=lights[i+1]; // remove light
			--light_cnt;											
			return true;

		} else return false;
		
	};

	public boolean	replaceLight (int index, Light light)
	// replaces a Light (especially useful, when utilizing moving lightsources)
	{
		if (checkRange(index))
		{
			Light tl = new Light(light);
			tl.dir = Coordinate.normalize(light.dir);
			lights[index]=tl;
			return true;
		} else return false;
	};

	public boolean replaceLight (int index, Coordinate pos, Coordinate dir)
	// replaces a Light (especially useful, when utilizing moving lightsources)
	{
		return replaceLight (index,new Light(pos,dir));
	};

	public void	removeAllLights ()
	// removes all lightsources
	{
		light_cnt = 0;
	};

	public void setFallOff (int fall)
	// sets the fallof value
	{
		fallOff = fall;	
	};

	public int getFallOff()
	// returns the fallof value
	{
		return fallOff;
	};

}