import java.awt.Graphics;	// graphics interface
import java.awt.Polygon;	// polygon interface
import java.awt.Color;		// color interface
import Coordinate;			// coordinates
import FastRotate;			// rotationobject
import NormalColor;			// colorobject
import OnePlane;

/**
 * OneBall.java: implementation example of interface class VectorObject
 *
 * @author Sven Sa 
 * @vesrion 09/06/97
 * @desc a Ball consisting of faces
 * @see VectorObject
 */
public class OneBall implements VectorObject
{
	//- Variablen --------------------------------------------------------
	private static final double pi = 3.14159265;		// ?!?

	private VectorObject [] faces;			// the faces of the plane
	private Coordinate	  rot;			// current rotation
	private Coordinate	  pos;			// current position

	private int hidden = VO_IS_NOT_HIDDEN;	// hidden flag

	private FastRotate fastrot;			// the rotation"machine"
	private NormalColor nc;				// the color calculation 

	private int facecnt = 0;

	/**
	 * Constructor
	 *
	 * @param rad radius of ball
	 * @param col color of ball
	 */
	public OneBall (int rad)
	{	
		createTables (rad,Color.white,8,null);// create internal tables
	} 

	/**
	 * Constructor
	 *
	 * @param rad radius of ball
	 * @param col color of ball
	 */
	public OneBall (int rad,Color col)
	{	
		createTables (rad,col,8,null);// create internal tables
	} 

	/**
	 * Constructor
	 *
	 * @param rad radius of ball
	 * @param col Color of ball
	 * @param pts amount of points used as base (>4)
	 */
	public OneBall (int rad,Color col, int pts, Color col2)
	{	
		createTables (rad,col,pts,col2);// create internal tables
	} 



	/**
	 * not used
	 * @return always -1 (object is final)
	 */
	public int addItem(VectorObject  obj)
	{
		return -1;
	}

	/**
	 * not used
	 * @return always false (object is final)
	 */
	public boolean deleteItem (int item)
	{
		return false;
	}

	/**
	 * retunrs number of subobjects
	 * @param void
	 * @return always 12 (cube consists of 12 faces)
	 */
	public int itemCount ( )
	{
		return facecnt;	
	}

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (int x_rot,int y_rot, int z_rot )
	{
		rot.x += x_rot;
		rot.y += y_rot;
		rot.z += z_rot;
	};

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (Coordinate rot)
	{
		rotate (rot.x,rot.y,rot.z);
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (int x_rot,int y_rot, int z_rot )
	{
		rot.x=x_rot;
		rot.y=y_rot;
		rot.z=z_rot;
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (Coordinate _rot)
	{
		rot = _rot;
	};

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 * @see VectorObject
	 */
	public Coordinate getRotation ( )
	{
		return rot;
	}

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (int x_mov,int y_mov, int z_mov )
	{
		pos.x += x_mov;
		pos.y += y_mov;
		pos.z += z_mov;
	};

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (Coordinate mov)
	{
		translate (mov.x,mov.y,mov.z);
	};

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (int x_mov,int y_mov, int z_mov )
	{
		pos.x = x_mov;
		pos.y = y_mov;
		pos.z = z_mov;
	};

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (Coordinate mov)
	{
		pos = mov;
	};

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 * @see VectorObject
	 */
	public Coordinate getTranslation ( )
	{
		return pos;
	}

	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 * @see VectorObject
	 */
	public void hide ( )
	{
		hidden = VO_IS_HIDDEN;	
	};
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void show ( )
	{
		hidden = VO_IS_NOT_HIDDEN;		
	};

	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void showAlways ( )
	{
		hidden = VO_NEVER_HIDDEN;		
	};

	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 * @see VectorObject
	 */
	public int isHidden ( )
	{
		return hidden;	
	};

	/**
	 * returns average depth of object
	 *
	 * @param void
	 * @return int depth of objectes
	 * @see VectorObject
	 */
	public int getDepth ( )
	{
		return pos.z;
	};

	/**
	 * draws an object on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 * @see VectorObject
	 */
	public void draw ( Graphics graph )
	{
	   if (hidden == VO_IS_NOT_HIDDEN || hidden == VO_NEVER_HIDDEN)
	   {
		for (int i=0;i<facecnt;++i)
		{
			faces[i].setRotation(rot);
			faces[i].setTranslation(pos);
			faces[i].draw(graph);
		}
	   }
	};

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @return always true
	 * @see VectorObject
	 */
	public boolean isVisible()
	{
		return true;
	}

	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces (1<=int<=6)
	 */
	public int getVisibleCount()
	{
		int ret=0;
		for (int i=0;i<facecnt;++i)ret+=faces[i].getVisibleCount();
		return ret;
	}

	/**
	 * creates the internal tables for cube object
	 *
	 * @param rad Radius of ball
	 * @param col color of ball
	 * @param pts amount of base points
	 * @return void	 
	 */
    private void createTables (int rad,Color col,int pts, Color col2)
	{
		rad /= 2;
		// initialize rotation
		fastrot = new FastRotate();

		// construct variables
		rot = new Coordinate();			
		pos = new Coordinate();			

		facecnt = (pts/2 )*pts;				// total amount of faces
		faces 		= new VectorObject[facecnt];				// Faces of ball
		facecnt=0;

		Coordinate[] c = new Coordinate[4];				// temp Coordinates
		Coordinate []points = new Coordinate[(pts+1)*2];

		
		for (int z=0; z<(pts/2-0  );++z)	// from top to bottom
		{
			// radius depending on top/bottom
			double zfact  =  ((double)(rad))*Math.sin ((double)(z+0)/(pts/2) * pi);
			double zfact2 =  ((double)(rad))*Math.sin ((double)(z+1)/(pts/2) * pi);

			// height depending on top/botton
			int yfact  = (int)(((double)(rad))*Math.cos ((double)(z+0)/(pts/2) * pi));
			int yfact2 = (int)(((double)(rad))*Math.cos ((double)(z+1)/(pts/2) * pi));

			// calculate a torus (not quite)
			for (int j=0; j<=pts;  ++j)
			{
				points[j      ]=new Coordinate((int)( zfact*Math.sin ((double)j*2/pts * pi)),
											   yfact,
											   (int)( zfact*Math.cos ((double)j*2/pts * pi)));

				points[j+1+pts]=new Coordinate((int)(zfact2*Math.sin ((double)j*2/pts * pi)),
											   yfact2, 
											   (int)( zfact2*Math.cos ((double)j*2/pts * pi)) );
			}

			// create faces bases on those points
			for (int j=0; j<pts;  ++j)
			{
				if (z==0)	// FIX: when z==0 c[0] == c[1] ==> normal vector goes wrong
				{
					c[0]=points[j+1      ];c[1]=points[j+pts+1+1]; 
					c[2]=points[j+pts+1];  c[3]=points[j        ]; 
				} else
				{
					c[0]=points[j		 ];c[1]=points[j+1	  ]; 
					c[2]=points[j+pts+1+1];c[3]=points[j+pts+1]; 
				}
				if ((col2!=null) && (facecnt+z)%2 ==0)	faces[facecnt]=new OnePlane(c,col);	// add face
				else									faces[facecnt]=new OnePlane(c,col2);

				facecnt++; 
			}
		}
	}
}


