import java.awt.Graphics;	// graphics interface
import java.awt.Color;		// color interface
import Coordinate;			// coordinates
import FastRotate;			// rotationobject
import NormalColor;			// colorobject
import OnePlane;			// a plane

/**
 * OneBar.java: implementation example of interface class VectorObject
 *
 * @author Sven Sa 
 * @version 09/06/97 
 * @desc a simple cube
 * @see VectorObject
 */
public class OneBar implements VectorObject
{
	//- Variablen --------------------------------------------------------
	private OnePlane [] faces;			// the faces of the plane
	private Coordinate	  rot;			// current rotation
	private Coordinate	  pos;			// current position

	private int hidden = VO_IS_NOT_HIDDEN;	// hidden flag

	private FastRotate fastrot;			// the rotation"machine"
	private NormalColor nc;				// the color calculation 

	/**
	 * Constructor (default color is Color.white)
	 *
	 * @param width width of bar
	 * @param height height of bar
	 * @param depth depth of bar
	 */
	public OneBar (int width, int height, int depth) 
	{	
		createTables (width,height,depth,Color.white);	// create internal tables
	} 

	/**
	 * Constructor
	 *
	 * @param width width of bar
	 * @param height height of bar
	 * @param depth depth of bar
	 * @param col color of bar
	 */
	public OneBar (int width, int height, int depth,Color col) 
	{	
		createTables (width,height,depth,col);// create internal tables
	} 
	
	/**
	 * not used
	 * @return always -1 (object is final)
	 */
	public int addItem(VectorObject  obj)
	{
		return -1;
	}

	/**
	 * not used
	 * @return always false (object is final)
	 */
	public boolean deleteItem (int item)
	{
		return false;
	}

	/**
	 * retunrs number of subobjects
	 * @param void
	 * @return always 24 (bar consists of 24 faces)
	 */
	public int itemCount ( )
	{
		return 12*faces[0].itemCount();	
	}

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (int x_rot,int y_rot, int z_rot )
	{
		rot.x += x_rot;
		rot.y += y_rot;
		rot.z += z_rot;
	};

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (Coordinate rot)
	{
		rotate (rot.x,rot.y,rot.z);
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (int x_rot,int y_rot, int z_rot )
	{
		rot.x=x_rot;
		rot.y=y_rot;
		rot.z=z_rot;
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (Coordinate _rot)
	{
		rot = _rot;
	};

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 * @see VectorObject
	 */
	public Coordinate getRotation ( )
	{
		return rot;
	}

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (int x_mov,int y_mov, int z_mov )
	{
		pos.x += x_mov;
		pos.y += y_mov;
		pos.z += z_mov;
	};

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (Coordinate mov)
	{
		translate (mov.x,mov.y,mov.z);
	};

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (int x_mov,int y_mov, int z_mov )
	{
		pos.x = x_mov;
		pos.y = y_mov;
		pos.z = z_mov;
	};

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (Coordinate mov)
	{
		pos = mov;
	};

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 * @see VectorObject
	 */
	public Coordinate getTranslation ( )
	{
		return pos;
	}

	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 * @see VectorObject
	 */
	public void hide ( )
	{
		hidden = VO_IS_HIDDEN;	
	};
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void show ( )
	{
		hidden = VO_IS_NOT_HIDDEN;		
	};

	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void showAlways ( )
	{
		hidden = VO_NEVER_HIDDEN;		
	};


	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 * @see VectorObject
	 */
	public int isHidden ( )
	{
		return hidden;	
	};

	/**
	 * returns average depth of object
	 *
	 * @param void
	 * @return int depth of objectktes
	 * @see VectorObject
	 */
	public int getDepth ( )
	{
		return pos.z;
	};

	/**
	 * draws an object on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 * @see VectorObject
	 */
	public void draw ( Graphics graph )
	{
	   if (hidden == VO_IS_NOT_HIDDEN)
	   {
		for (int i=0;i<12;++i)
		{
			faces[i].setRotation(rot);
			faces[i].setTranslation(pos);
			faces[i].draw(graph);
		}
	   }
	};

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @return always true
	 * @see VectorObject
	 */
	public boolean isVisible()
	{
		return true;
	}

	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces (1<=int<=6)
	 */
	public int getVisibleCount()
	{
		int ret=0;
		for (int i=0;i<12;++i)ret+=faces[i].getVisibleCount();
		return ret;
	}

	/**
	 * creates the internal tables for bar object
	 *
	 * @param dim dimension of bar
	 * @param col color of faces
	 * @return void	 
	 */
    private void createTables (int width, int height, int depth,Color col)
	{
		// initialize rotation
		fastrot = new FastRotate();

		// construct variables
		rot = new Coordinate();			
		pos = new Coordinate();			

		Coordinate []points = new Coordinate[16];
		points[ 0] = new Coordinate(-1*width/2,-1*height/2,-1*depth/2); // Points of Bar
		points[ 1] = new Coordinate( 1*width/2,-1*height/2,-1*depth/2);
		points[ 2] = new Coordinate( 1*width/2, 1*height/2,-1*depth/2);
		points[ 3] = new Coordinate(-1*width/2, 1*height/2,-1*depth/2);

		points[ 4] = new Coordinate(-1*width/2,-1*height/2, 1*depth/2);
		points[ 5] = new Coordinate( 1*width/2,-1*height/2, 1*depth/2);
		points[ 6] = new Coordinate( 1*width/2, 1*height/2, 1*depth/2);
		points[ 7] = new Coordinate(-1*width/2, 1*height/2, 1*depth/2);

		points[ 8] = new Coordinate(-1*width/2,-1*height/2, 0);
		points[ 9] = new Coordinate( 1*width/2,-1*height/2, 0);
		points[10] = new Coordinate( 1*width/2, 1*height/2, 0);
		points[11] = new Coordinate(-1*width/2, 1*height/2, 0);

		points[12] = new Coordinate(         0,-1*height/2,-1*depth/2);
		points[13] = new Coordinate(         0,-1*height/2, 1*depth/2);
		points[14] = new Coordinate(         0, 1*height/2, 1*depth/2);
		points[15] = new Coordinate(         0, 1*height/2,-1*depth/2);

		Coordinate[] c = new Coordinate[4];			// temp Coordinates
		faces 		= new OnePlane[12];				// Faces of Cube

		c[0]=points[ 8]; c[1]=points[ 9]; c[2]=points[ 1];c[3]=points[ 0]; faces[ 0]=new OnePlane(c,col);
		c[0]=points[ 4]; c[1]=points[ 5]; c[2]=points[ 9];c[3]=points[ 8]; faces[ 1]=new OnePlane(c,col);
		c[0]=points[ 9]; c[1]=points[10]; c[2]=points[ 2];c[3]=points[ 1]; faces[ 2]=new OnePlane(c,col);
		c[0]=points[ 5]; c[1]=points[ 6]; c[2]=points[10];c[3]=points[ 9]; faces[ 3]=new OnePlane(c,col);
		c[0]=points[11]; c[1]=points[ 8]; c[2]=points[ 0];c[3]=points[ 3]; faces[ 4]=new OnePlane(c,col);
		c[0]=points[ 7]; c[1]=points[ 4]; c[2]=points[ 8];c[3]=points[11]; faces[ 5]=new OnePlane(c,col);
		c[0]=points[ 3]; c[1]=points[ 2]; c[2]=points[10];c[3]=points[11]; faces[ 6]=new OnePlane(c,col);
		c[0]=points[11]; c[1]=points[10]; c[2]=points[ 6];c[3]=points[ 7]; faces[ 7]=new OnePlane(c,col);

		c[0]=points[ 0]; c[1]=points[12]; c[2]=points[15];c[3]=points[ 3]; faces[ 8]=new OnePlane(c,col);
		c[0]=points[12]; c[1]=points[ 1]; c[2]=points[ 2];c[3]=points[15]; faces[ 9]=new OnePlane(c,col);
		c[0]=points[ 5]; c[1]=points[13]; c[2]=points[14];c[3]=points[ 6]; faces[10]=new OnePlane(c,col);
		c[0]=points[13]; c[1]=points[ 4]; c[2]=points[ 7];c[3]=points[14]; faces[11]=new OnePlane(c,col);
	}
}
