
import java.awt.Graphics;	// graphics interface
import java.awt.Polygon;	// polygon interface
import java.awt.Color;		// color interface
import Coordinate;			// coordinates
import FastRotate;			// rotationobject
import NormalColor;			// colorobject

/**
 * OneOval.java:		implementation example of interface class VectorObject
 *
 * @author Sven Sa 
 * @version 09/06/97
 * @desc    one single oval
 */
public class OneOval implements VectorObject
{
	private int dist =  512;	// Default Abstand des Objektes DEBUG!!
	private int eye  =  512;	// Augenabstand
	private int mult = 2048;  // Berechnungsgenauigkeit
	private int x_off = 320;
	private int y_off = 200;

	//- Variablen --------------------------------------------------------
 	private Coordinate [] punkte;		// the points of the plane
	private Coordinate [] rot_punkte;	// the rotated points	 (4th is normal vector!)
	private Polygon		  per_punkte;	// the perpectived points
	private Coordinate	  rot;			// current rotation
	private Coordinate	  pos;			// current position
	private int hidden = VO_IS_NOT_HIDDEN;
	private int size=0;
	private boolean sameColor;
	private Color defColor;

	private FastRotate fastrot;			// the rotation"machine"
	private NormalColor nc;				// the color calculation 

	/**
	 * initializes one oval
	 *
	 * @param _size size of oval
	 * @param col color of oval
	 */
	public OneOval (int _size, Color col, boolean _sameColor)
	{
		sameColor = _sameColor;
		defColor = col;

		size = _size/2;
		// turn on rotation
		fastrot = new FastRotate();
		nc = new NormalColor (col);

		// reserve memory
		punkte	   = new Coordinate[2];
		rot_punkte = new Coordinate[2];
		per_punkte = new Polygon();		// must be initialized
		per_punkte.addPoint	(0,0);
		per_punkte.addPoint	(0,0);

		rot = new Coordinate();			// current rotation
		pos = new Coordinate();			// current position

		punkte[0] = new Coordinate (_size,_size,0); // height, width
		punkte[1] = Coordinate.normalize (new Coordinate (0,0,-Coordinate.NORMAL_BASE));
		
		calculate();
	}
	
	/**
	 * not used
	 * @return always -1 (object is final)
	 */
	public int addItem(VectorObject  obj)
	{
		return -1; // Objekt ist final
	}

	/**
	 * not used
	 * @return always false (object is final)
	 */
	public boolean deleteItem (int item)
	{
		return false; // kann keine Objekt entfernen
	}

	/**
	 * retunrs number of subobjects
	 * @param void
	 * @return always 1 (face has always one face)
	 */
	public int itemCount ( )
	{
		return 1;	
	}

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (int x_rot,int y_rot, int z_rot )
	{
		rot.x += x_rot;
		rot.y += y_rot;
		rot.z += z_rot;
	};

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (Coordinate rot)
	{
		rotate (rot.x,rot.y,rot.z);
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (int x_rot,int y_rot, int z_rot )
	{
		rot.x=x_rot;
		rot.y=y_rot;
		rot.z=z_rot;
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (Coordinate _rot)
	{
		rot = _rot;
	};

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 * @see VectorObject
	 */
	public Coordinate getRotation ( )
	{
		return rot;
	}

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (int x_mov,int y_mov, int z_mov )
	{
		pos.x += x_mov;
		pos.y += y_mov;
		pos.z += z_mov;
	};

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (Coordinate mov)
	{
		translate (mov.x,mov.y,mov.z);
	};

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (int x_mov,int y_mov, int z_mov )
	{
		pos.x = x_mov;
		pos.y = y_mov;
		pos.z = z_mov;
	};

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (Coordinate mov)
	{
		pos = mov;
	};

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 * @see VectorObject
	 */
	public Coordinate getTranslation ( )
	{
		return pos;
	}

	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 * @see VectorObject
	 */
	public void hide ( )
	{
		hidden = VO_IS_HIDDEN;	
	};
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void show ( )
	{
		hidden = VO_IS_NOT_HIDDEN;		
	};

	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void showAlways ( )
	{
		hidden = VO_NEVER_HIDDEN;		
	};

	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 * @see VectorObject
	 */
	public int isHidden ( )
	{
		return hidden;	
	};

	/**
	 * returns average depth of object
	 *
	 * @param void
	 * @return int depth of object
	 * @see VectorObject
	 */
	public int getDepth ( )
	{
		return pos.z;
	};

	/**
	 * draws an oval on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 * @see VectorObject
	 */
	public void draw ( Graphics graph )
	{
	   if (hidden == VO_IS_NOT_HIDDEN)
	   {
		calculate();
		if (isVisible() || hidden == VO_NEVER_HIDDEN)		// only draw if visible or NEVER HIDDEN
		{
			if (sameColor)	graph.setColor (defColor);
			else			graph.setColor(nc.lightning(pos,rot_punkte[1]));	// calculate color
			graph.fillOval( pos.x + x_off-size,
							pos.y + y_off-size,
							per_punkte.ypoints[0],
							per_punkte.xpoints[0]);	// draw oval
		}
	   } // is not hidden
	};

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @returns TRUE if perspectived normal vector has positive z-axis value or<br>
     *				 hidden flag equals VO_NEVER_HIDDEN
	 * @see VectorObject
	 */
	public boolean isVisible()
	// checks if the z-component of perspectived normal vector is positive
	{
		return (hidden == VO_NEVER_HIDDEN) ||
				(rot_punkte[1].z<0);
	}

	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces (0<=int<=1)
	 */
	public int getVisibleCount()
	{
		return (isVisible())?1:0;
	}

	/**
	 * does the internal rotation calculation
	 *
	 * @param void
	 * @return void	 
	 */
	private void calculate()
	{
 		fastrot.setRotation (rot);							// set position
		fastrot.calculate(punkte,rot_punkte,2);				// rotate points (incl. normal)
	
		for (int i=0; i<2; ++i)								// perspective points(NOT normal!)
		{
    		per_punkte.xpoints[i]=(((rot_punkte[i].x)*mult)/((rot_punkte[i].z+pos.z)+dist)*eye)/mult;
			per_punkte.ypoints[i]=(((rot_punkte[i].y)*mult)/((rot_punkte[i].z+pos.z)+dist)*eye)/mult;
		}
	}
}
