import java.awt.Graphics;	// graphics interface
import java.awt.Polygon;	// polygon interface
import java.awt.Color;		// color interface
import Coordinate;			// coordinates
import FastRotate;			// rotationobject
import NormalColor;			// colorobject
import OneFace;

/**
 * OnePyramid.java: implementation example of interface class VectorObject
 *
 * @author Sven Sa 
 * @vesrion 09/06/97
 * @desc a simple cube
 * @see VectorObject
 */
public class OnePyramid implements VectorObject
{
	//- Variablen --------------------------------------------------------
	private OneFace [] faces;			// the faces of the plane
	private Coordinate	  rot;			// current rotation
	private Coordinate	  pos;			// current position

	private int hidden = VO_IS_NOT_HIDDEN;	// hidden flag

	private FastRotate fastrot;			// the rotation"machine"
	private NormalColor nc;				// the color calculation 

	/**
	 * Constructor
	 *
	 * @param size size of borders
	 */
	public OnePyramid (int size)
	{	
		Color [] cols = new Color[4];
		for (int i=0;i<4;++i) cols [i] = Color.white;	// do some default colors
		cols[ 0] = Color.red;
		cols[ 1] = Color.green;
		cols[ 2] = Color.blue;
		cols[ 3] = Color.magenta;
		createTables (size,cols);						// and create internal tables
	} 

	/**
	 * Constructor
	 *
	 * @param size size of borders
	 * @param col color of the cube (all faces)
	 */
	public OnePyramid (int size, Color col)
	{
		Color [] cols = new Color[4];
		for (int i=0;i<4;++i) cols [i] = col;			// create color table
		createTables (size,cols);						// and create internal tables
	} 

	/**
	 * not used
	 * @return always -1 (object is final)
	 */
	public int addItem(VectorObject  obj)
	{
		return -1;
	}

	/**
	 * not used
	 * @return always false (object is final)
	 */
	public boolean deleteItem (int item)
	{
		return false;
	}

	/**
	 * retunrs number of subobjects
	 * @param void
	 * @return always 12 (pyramid consists of 4 faces)
	 */
	public int itemCount ( )
	{
		return 4;	
	}

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (int x_rot,int y_rot, int z_rot )
	{
		rot.x += x_rot;
		rot.y += y_rot;
		rot.z += z_rot;
	};

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 * @see VectorObject
	 */
	public void rotate (Coordinate rot)
	{
		rotate (rot.x,rot.y,rot.z);
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (int x_rot,int y_rot, int z_rot )
	{
		rot.x=x_rot;
		rot.y=y_rot;
		rot.z=z_rot;
	};

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setRotation (Coordinate _rot)
	{
		rot = _rot;
	};

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 * @see VectorObject
	 */
	public Coordinate getRotation ( )
	{
		return rot;
	}

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (int x_mov,int y_mov, int z_mov )
	{
		pos.x += x_mov;
		pos.y += y_mov;
		pos.z += z_mov;
	};

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void translate (Coordinate mov)
	{
		translate (mov.x,mov.y,mov.z);
	};

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (int x_mov,int y_mov, int z_mov )
	{
		pos.x = x_mov;
		pos.y = y_mov;
		pos.z = z_mov;
	};

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 * @see VectorObject
	 */
	public void setTranslation (Coordinate mov)
	{
		pos = mov;
	};

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 * @see VectorObject
	 */
	public Coordinate getTranslation ( )
	{
		return pos;
	}

	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 * @see VectorObject
	 */
	public void hide ( )
	{
		hidden = VO_IS_HIDDEN;	
	};
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void show ( )
	{
		hidden = VO_IS_NOT_HIDDEN;		
	};

	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 * @see VectorObject
	 */
	public void showAlways ( )
	{
		hidden = VO_NEVER_HIDDEN;		
	};

	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 * @see VectorObject
	 */
	public int isHidden ( )
	{
		return hidden;	
	};

	/**
	 * returns average depth of object
	 *
	 * @param void
	 * @return int depth of objectktes
	 * @see VectorObject
	 */
	public int getDepth ( )
	{
		return pos.z;
	};

	/**
	 * draws an object on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 * @see VectorObject
	 */
	public void draw ( Graphics graph )
	{
	   if (hidden == VO_IS_NOT_HIDDEN)
	   {
		for (int i=0;i<4;++i)
		{
			faces[i].setRotation(rot);
			faces[i].setTranslation(pos);
			faces[i].draw(graph);
		}
	   }
	};

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @return always true
	 * @see VectorObject
	 */
	public boolean isVisible()
	{
		return true;
	}

	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces (1<=int<=6)
	 */
	public int getVisibleCount()
	{
		int ret=0;
		for (int i=0;i<4;++i)ret+=faces[i].getVisibleCount();
		return ret;
	}

	/**
	 * creates the internal tables for cube object
	 *
	 * @param size size of borders
	 * @param cols[] 12 colors defining the face colors
	 * @return void	 
	 */
    private void createTables (int size,Color cols[])
	{
		// initialize rotation
		fastrot = new FastRotate();

		// construct variables
		rot = new Coordinate();			
		pos = new Coordinate();			

		Coordinate []points = new Coordinate[4];
		points[0] = new Coordinate(        0,-size*57/100,               0); // top
		points[1] = new Coordinate(        0, size*2886/10000, size*5773/10000);
		points[2] = new Coordinate( size*1/2, size*2886/10000,-size*2886/10000);
		points[3] = new Coordinate(-size*1/2, size*2886/10000,-size*2886/10000);

		Coordinate[] c = new Coordinate[3];			// temp Coordinates
		faces 		= new OneFace[4];				// Faces of Cube

		c[0]=points[0]; c[1]=points[1]; c[2]=points[2]; faces[ 0]=new OneFace(c,cols[ 0]);
		c[0]=points[0]; c[1]=points[2]; c[2]=points[3]; faces[ 1]=new OneFace(c,cols[ 1]);
		c[0]=points[0]; c[1]=points[3]; c[2]=points[1]; faces[ 2]=new OneFace(c,cols[ 2]);
		c[0]=points[1]; c[1]=points[3]; c[2]=points[2]; faces[ 3]=new OneFace(c,cols[ 3]);

	}
}


