import Coordinate;			// coordinates

/**
 * RotationEngine.java:	interface-class   
 *						definines the interface that every rotation engine
 *						has to implement
 * @author Sven Sa 
 * @version V1.0 08/06/97
 * @desc rotatate coordinates in a normal vector system
 */
public interface  RotationEngine
{
	/**
	 * Does the calculation for rotation
	 *
	 * @param x_rot coordinate x-portion
	 * @param y_rot coordinate y-portion
	 * @param z_rot coordinate z-portion
	 * @return coord - the rotated coordinates
	 */
	public abstract Coordinate calculate (int x_rot,int y_rot, int z_rot );

	/**
	 * Does the calculation for rotation
	 *
	 * @param rot coordinate portion on xyz-axis
	 * @return coord - the rotated coordinates
	 */
	public abstract Coordinate calculate (Coordinate rot);

	/**
	 * Does the calculation for an array of coordinates
	 *
	 * @param src[] source buffer of coordinates
	 * @param dst[] destination buffer of coordinates
	 * @param cnt amount of coordinates in buffer
	 *
	 * @return the rotated coordinates in dest.buffer zu einer Koordinate
	 */
	public abstract void calculate (Coordinate src[],Coordinate dst[],int cnt);

	/**
	 * set relative rotation
	 *
	 * @param x_rot rotation on x-axis
	 * @param y_rot rotation on y-axis
	 * @param z_rot rotation on z-axis
	 * @return void
	 */
	public abstract void rotate (int x_rot,int y_rot, int z_rot );

	/**
	 * set the relative rotation
	 *
	 * @param rot rotation on xyz-axis
	 * @return void
	 */
	public abstract void rotate (Coordinate rot);

	/**
	 * sets the absolute rotation
	 *
	 * @param x_rot rotation on x-axis
	 * @param y_rot rotation on y-axis
	 * @param z_rot rotation on z-axis
	 * @return void
	 */
	public abstract void setRotation (int x_rot,int y_rot, int z_rot );

	/**
	 * sets the absolute rotation
	 *
	 * @param rot rotation on xyz-axis
	 * @return void
	 */
	public abstract void setRotation (Coordinate rot);

	/**
	 * returns the current rotation
	 *
	 * @param void
	 * @return rot the rotation on xyz-axis
	 */
	public abstract Coordinate getRotation ( );
}
