//******************************************************************************
// Satellite.java:	defines a vector world that rotatates different satellites
//					around a fixed centrum
//
// @author:  Sven Sa 
// @vesrion: 20/05/97
//******************************************************************************

import java.applet.*;
import java.awt.*;
import java.util.Random;

//import VectorWorld;			// a whole vector world
import VectorObject;		// one vector object
import NormalColor;			// normal Color calculation
import Coordinate;			// coordinate class
import Light;				// light class
import OnePyramid;			// one simple pyramid
import OnePlane;			// one plane (consisting out of two faces)
import OneBar;				// one bar

public class Satellite 
{
	//- Variablen ---------------------------------------------------------------
	private NormalColor nc;
	private VectorObject cube;
	private OneOval light_oval,light_oval2;
	private Light light	 = new Light (new Coordinate (0,0,-1024),new Coordinate (0,0,1024)),rot_light =new Light(light);
	private Light light2 = new Light (new Coordinate (0,0,-1024),new Coordinate (0,0,1024)),rot_light2=new Light(light2);
	private FastRotate fastrot	= new FastRotate();
	private FastRotate fastrot2	= new FastRotate();

	private int sat_amount;
	private FastRotate	[]sat_rot;
	private Coordinate  []sat_pos;
	private Coordinate  []rot_pos;
	private Coordinate  []rot_rot;
	private Coordinate  []rot_rot2;
	private Random rand = new Random();

	private int wobj_cnt;
	private VectorObject []world;
	private int dlist [];

	int faces=0,totfaces=0;
	boolean lightrot=false;

    /**
     * constructor
	 */
	public Satellite()
	{
		sat_amount = 10;						// amount of satellites
		createTables(0);
	}

    /**
     * constructor
	 */
	public Satellite(int mainsw, int sats,boolean lrot)
	{
		sat_amount = sats;						// amount of satellites
		lightrot = lrot;
		createTables(mainsw);
//					cube.setRotation (90*4,45*4,0);
	}


	/**
	 * draws an satellite world on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 */
	public void draw ( Graphics graph )
	{

		if (lightrot)
		{
			fastrot.rotate (-3*4,-2*4,0);						// do some light rotation
			rot_light.dir = fastrot.calculate (light.dir);
			nc.replaceLight (0,rot_light);
			light_oval.setTranslation (-rot_light.dir.x/6,-rot_light.dir.y/6,-rot_light.dir.z/6);

			fastrot2.rotate (6,10,0);						// do some light rotation
			rot_light2.dir = fastrot2.calculate (light2.dir);
			nc.replaceLight (1,rot_light2);
			light_oval2.setTranslation (-rot_light2.dir.x/6,-rot_light2.dir.y/6,-rot_light2.dir.z/6);

		}

		cube.rotate (2*4,1*4,0);

		for (int i=0;i<sat_amount; ++i)				// satellites
		{
		  	sat_rot[i].rotate (rot_rot[i]);					
		  	rot_pos[i] = sat_rot[i].calculate (sat_pos[i]);
		  	world [i].setTranslation(rot_pos[i]);
		  	world [i].rotate(rot_rot2[i]);
		}
		
		sort();
		faces=0;
		for (int i=0;i<wobj_cnt;++i)
		{
			world[dlist[i]].draw(graph);
			faces+=world[dlist[i]].getVisibleCount();
		}
	}; // draw

	/**
	 * Amount of faces in this object (of last frame)
	 *
	 * @param	none
	 * @return	int	amount of faces
	 */
	public int itemCount ()
	{
		return totfaces;
	};

	/**
	 * returns the amount of visible faces in this object
	 *
	 * @param void
	 * @return int - amount of visible faces
	 */
	public int getVisibleCount()
	{
		return faces;
	};

	/**
	 * creates the internal tables
	 *
	 * @param
	 * @return created internal tables
	 */
	private void createTables (int mainsw)
	{
		// reserve memory and stuff
		sat_rot		= new FastRotate  [sat_amount];
		sat_pos		= new Coordinate  [sat_amount];
		rot_pos		= new Coordinate  [sat_amount];
		rot_rot		= new Coordinate  [sat_amount];
		rot_rot2	= new Coordinate  [sat_amount];
			
		wobj_cnt = sat_amount+3;				    // total amount of objects
		world	 = new VectorObject [wobj_cnt];

		dlist = new int[wobj_cnt];				// display list
		for (int i=0;i<wobj_cnt;++i)dlist[i]=i;		

		switch  (mainsw)
		{
			default:
			case 0: {cube		= new OneCube(40,Color.lightGray);break;}
			case 1: {
						int size = 150;
						Coordinate []points = new Coordinate[8];
						points[0] = new Coordinate(-1*size/2,-1*size/2,0); // Points of Face
						points[1] = new Coordinate( 1*size/2,-1*size/2,0);
						points[2] = new Coordinate( 1*size/2, 1*size/2,0);
						cube		= new OneFace(points,Color.gray);
						cube.showAlways();
						break;
					}
			case 2: {cube		= new OnePyramid(150,Color.lightGray);break;}
			case 3: {cube		= new OneBall(250,Color.blue,20,Color.red);break;}
			case 4: {cube		= new OneCube(150,Color.lightGray);break;}
			case 5: {cube		= new General(150);break;}

		}
		
		light_oval	= new OneOval(30,Color.yellow,true);	// light Cube
		light_oval2	= new OneOval(30,Color.yellow,true);	// light Cube

		for (int i=0;i<sat_amount; ++i)				// satellites
		{
		   Color col;
		   switch (rand.nextInt()%3)
		   {
 				case -2: col = Color.red;		break;
				case -1: col = Color.green;		break;
				case  0: col = Color.blue;		break;
				case  1: col = Color.cyan;		break;
				case  2: col = Color.magenta;	break;
				default: col = Color.white;
		   }
		   int sw = rand.nextInt()%2+2;
		   if (sw>4) sw =4;
				   
		   switch (sw)
		   {
				case  0: {
						   System.out.println ("Adding Cube");
						   OneCube c = new OneCube (50,col);
						   world [i] = c; 
						   break;
						 }
				case  1: {
						   System.out.println ("Adding Pyramid");	
						   OnePyramid p = new OnePyramid (50,col);
						   world [i] = p;
						   break;
						 }
				case  2: {
							System.out.println ("Adding Bar");
							OneBar b = new OneBar (15,20,50,col);
							world [i] = b;
							break;
						 }
				case  3: {
							System.out.println ("Adding Ball");
							OneBall b = new OneBall (50,col,16,null);
							world [i] = b;
							break;
						 }
				default:
	  			case 4: {
							System.out.println ("Adding Face");
							int size = 25;
							Coordinate []points = new Coordinate[8];
							points[0] = new Coordinate(-1*size/2,-1*size/2,0); // Points of Cube
							points[1] = new Coordinate( 1*size/2,-1*size/2,0);
							points[2] = new Coordinate( 1*size/2, 1*size/2,0);
							points[3] = new Coordinate(-1*size/2, 1*size/2,0);
						    OnePlane p = new OnePlane (points,col);
							p.showAlways();
						    world [i] = p;
							break;
						 }
		   }


		   sat_pos	 [i] = new Coordinate (70+rand.nextInt()%150, // random start position
										   70+rand.nextInt()%150,
										   70+rand.nextInt()%150);
		   rot_pos	 [i] = new Coordinate (sat_pos[i]);
		   rot_rot   [i] = new Coordinate (rand.nextInt()%16,rand.nextInt()%16,0);
		   rot_rot2  [i] = new Coordinate (rand.nextInt()%16,rand.nextInt()%16,0);
		   sat_rot   [i] = new FastRotate();
		} // create Satellites


		world[sat_amount  ] = cube;
		if (lightrot)
		{
			world[sat_amount+1] = light_oval;
			world[sat_amount+2] = light_oval2;
		} else wobj_cnt-=2;
	
		nc = new NormalColor ();					// lightsourcing
  		nc.addLight (light);
		if (lightrot)nc.addLight(light2);
		for (int i=0;i<wobj_cnt;++i)
		{
			totfaces+=world[dlist[i]].itemCount();
		};
	} // createTables

	private void sort()
	// based on bubble sort
	{
		int temp;
		int i=0;
		boolean b=true;
		while (b && i<wobj_cnt)
		{
			b=false;
			for (int j=i+1;j<wobj_cnt;++j)
			{
				if (world[dlist[j-1]].getDepth()<world[dlist[j]].getDepth() )
				{
					temp =   dlist[j-1];  dlist[j-1]= dlist[j];   dlist[j]=temp;
					b=true;
				}
			}
			i++;
		}
	}

} // class Satellite



