import Coordinate;			// the coordinate class
import java.awt.Graphics;	// graphics interface

public interface  VectorObject
{
	/**
	* VectorObject.java:	interface-Klasse
	*						definnes the interface that every vector object
	*						has to implement
	*
	* @author:		Sven Sa 
	* @version:	V1.0 09/06/98
	*/

	public final static int VO_FINAL		  = 1;	// is object final?
	public final static int VO_HAS_SUBOBJECTS = 2;	// any subobjects defined?
	public final static int VO_IS_VISBLE      = 3;	// is visible?
	public final static int VO_IS_HIDDEN	  = 4;	// object is not shown
	public final static int VO_IS_NOT_HIDDEN  = 5;	// object is shown
	public final static int VO_NEVER_HIDDEN	  = 6;	// object is always show
	
	int hidden = VO_IS_NOT_HIDDEN;

    /**
     * Adds another object to this object.
     *
     * @param     obj	object to be added
     * @return    -1	- object has not been added (no further objects or "VO_FINAL")<br>
	 *			  int - no. of added object
	 */
	public abstract int addItem(VectorObject  obj);

	/**
	 * Removes an object from this object.
	 *
	 * @param	item	number of object to be removed
	 * @return	true  -	 object has not been removed<br>
	 *			false -	 object has been removed
	 */
	public abstract boolean deleteItem (int item);

	/**
	 * amount of (sub-)objects.
	 *
	 * @param	none
	 * @return	int	amount of subobjects in this object
	 */
	public abstract int itemCount ( );

	/**
	 * rotates object (relative to current position)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 */
	public abstract void rotate (int x_rot,int y_rot, int z_rot );

	/**
	 * rotates object (relative to current position)
	 *
	 * @param rot rotation around xyz-axis'
	 * @return void
	 */
	public abstract void rotate (Coordinate rot);

	/**
	 * rotates object (absolut)
	 *
	 * @param x_rot	rotation around x-axis
	 * @param y_rot rotation around y-axis
	 * @param z_rot rotation around z-axis
	 * @return void
	 */
	public abstract void setRotation (int x_rot,int y_rot, int z_rot );

	/**
	 * rotates object (absolut)
	 *
	 * @param rot rotation around xyz-axis
	 * @return void
	 */
	public abstract void setRotation (Coordinate rot);

	/**
	 * returns current rotation of object
	 *
	 * @param void
	 * @return Coordinate current rotation
	 */
	public abstract Coordinate getRotation ( );

	/**
	 *	translates an object relative to current position
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 */
	public abstract void translate (int x_mov,int y_mov, int z_mov );

	/**
	 * translate object relative to current position
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 */
	public abstract void translate (Coordinate mov);

	/**
	 * translate object (absolut)
	 *
	 * @param x_mov	translation along x-axis
	 * @param y_mov	translation along y-axis
	 * @param z_mov	translation along z-axis
	 * @return void
	 */
	public abstract void setTranslation (int x_mov,int y_mov, int z_mov );

	/**
	 * setTranslation:		verschiebt das Objekt (absolut zum (eigenen) Ursprung)
	 *
	 * @param mov translation along xyz-axis
	 * @return void
	 */
	public abstract void setTranslation (Coordinate mov);

	/**
	 * returns the current translation(position) of object
	 *
	 * @param void
	 * @return coord - the coordinates
	 */
	public abstract Coordinate getTranslation ( );
	
	/**
	 * sets hidden flag (object is not draw)
	 *
	 * @param void
	 * @return void
	 */
	public abstract void hide ( );
	
	/**
	 * resets the hidden flag (object is draw)
	 *
	 * @param void
	 * @return void	void
	 */
	public abstract void show ( );

	/**
	 * resets the hidden flag (object is always drawn)
	 *
	 * @param void
	 * @return void	void
	 */
	public abstract void showAlways ( );


	/**
     * returns the hidden flag
	 *
	 * @param void
	 * @return	VO_IS_HIDDEN	 (object is <b>not</b> draw)<br>
	 *			VO_IS_NOT_HIDDEN (object is draw)
	 */
	public abstract int isHidden ( );

	/**
	 * returns averaged depth of object
	 *
	 * @param void
	 * @return int depth of objectktes
	 */
	public abstract int getDepth ( );

	/**
	 * draws an object on given canvas
	 *
	 * @param graph	canvas on which object should be drawn
	 * @return void
	 */
	public abstract void draw ( Graphics graph );

	/**
	 * returns if object is visible
	 * 
	 * @param void
	 * @return true object is visible<br>
	 *		   false object is not drawn
	 */
	public abstract boolean isVisible ();

	/**
	 * returns the amount of visible faces in object
	 *
	 * @param void
	 * @return int - amount of visible faces
	 */
	public abstract int getVisibleCount();
}
